@@ -184,10 +184,10 @@ def prep_trans_tar(opts, file_client, chunks, file_refs, pillar=None, id_=None):
     if id_ is None:
         id_ = ''
     try:
-        cachedir = os.path.join(opts['cachedir'], 'salt-ssh', id_).rstrip(os.sep)
+        cachedir = os.path.join('salt-ssh', id_).rstrip(os.sep)
     except AttributeError:
         # Minion ID should always be a str, but don't let an int break this
-        cachedir = os.path.join(opts['cachedir'], 'salt-ssh', str(id_)).rstrip(os.sep)
+        cachedir = os.path.join('salt-ssh', str(id_)).rstrip(os.sep)
 
     for saltenv in file_refs:
         # Location where files in this saltenv will be cached
@@ -217,7 +217,7 @@ def prep_trans_tar(opts, file_client, chunks, file_refs, pillar=None, id_=None):
                     files = ''
                 if files:
                     for filename in files:
-                        fn = filename[len(cache_dest):].strip('/')
+                        fn = filename[len(file_client.get_cachedir(cache_dest)):].strip('/')
                         tgt = os.path.join(
                                 env_root,
                                 short,
