@@ -24,26 +24,37 @@ import getpass
 import salt.ext.six as six
 from salt.ext.six.moves import zip  # pylint: disable=import-error,redefined-builtin
 try:
-    from Crypto.Cipher import AES, PKCS1_OAEP
-    from Crypto.Hash import SHA
-    from Crypto.PublicKey import RSA
-    from Crypto.Signature import PKCS1_v1_5
-    # let this be imported, if possible
-    import Crypto.Random  # pylint: disable=W0611
+    from Cryptodome.Cipher import AES, PKCS1_OAEP
+    from Cryptodome.Hash import SHA
+    from Cryptodome.PublicKey import RSA
+    from Cryptodome.Signature import PKCS1_v1_5
+    import Cryptodome.Random  # pylint: disable=W0611
+    CDOME = True
 except ImportError:
-    # No need for crypt in local mode
-    pass
+    CDOME = False
+if not CDOME:
+    try:
+        from Crypto.Cipher import AES, PKCS1_OAEP
+        from Crypto.Hash import SHA
+        from Crypto.PublicKey import RSA
+        from Crypto.Signature import PKCS1_v1_5
+        # let this be imported, if possible
+        import Crypto.Random  # pylint: disable=W0611
+    except ImportError:
+        # No need for crypt in local mode
+        pass
 
 # Import salt libs
 import salt.defaults.exitcodes
 import salt.utils
 import salt.payload
 import salt.transport.client
+import salt.transport.frame
 import salt.utils.rsax931
 import salt.utils.verify
 import salt.version
 from salt.exceptions import (
-    AuthenticationError, SaltClientError, SaltReqTimeoutError, SaltSystemExit
+    AuthenticationError, SaltClientError, SaltReqTimeoutError
 )
 
 import tornado.gen
@@ -67,7 +78,7 @@ def dropfile(cachedir, user=None):
         if os.path.isfile(dfn) and not os.access(dfn, os.W_OK):
             os.chmod(dfn, stat.S_IRUSR | stat.S_IWUSR)
         with salt.utils.fopen(dfn, 'wb+') as fp_:
-            fp_.write('')
+            fp_.write(b'')
         os.chmod(dfn, stat.S_IRUSR)
         if user:
             try:
@@ -174,7 +185,7 @@ def gen_signature(priv_path, pub_path, sign_path):
                   'remove it first and try again'.format(sign_path))
     else:
         with salt.utils.fopen(sign_path, 'wb+') as sig_f:
-            sig_f.write(mpub_sig_64)
+            sig_f.write(salt.utils.to_bytes(mpub_sig_64))
         log.trace('Wrote signature to {0}'.format(sign_path))
     return True
 
@@ -365,14 +376,15 @@ class AsyncAuth(object):
         :rtype: Auth
         '''
         self.opts = opts
-        self.token = Crypticle.generate_key_string()
+        if six.PY2:
+            self.token = Crypticle.generate_key_string()
+        else:
+            self.token = salt.utils.to_bytes(Crypticle.generate_key_string())
         self.serial = salt.payload.Serial(self.opts)
         self.pub_path = os.path.join(self.opts['pki_dir'], 'minion.pub')
         self.rsa_path = os.path.join(self.opts['pki_dir'], 'minion.pem')
-        if 'syndic_master' in self.opts:
+        if self.opts['__role'] == 'syndic':
             self.mpub = 'syndic_master.pub'
-        elif 'alert_master' in self.opts:
-            self.mpub = 'monitor_master.pub'
         else:
             self.mpub = 'minion_master.pub'
         if not os.path.isfile(self.pub_path):
@@ -419,6 +431,13 @@ class AsyncAuth(object):
                self._authenticate_future.done() and \
                self._authenticate_future.exception() is None
 
+    def invalidate(self):
+        if self.authenticated:
+            del self._authenticate_future
+            key = self.__key(self.opts)
+            if key in AsyncAuth.creds_map:
+                del AsyncAuth.creds_map[key]
+
     def authenticate(self, callback=None):
         '''
         Ask for this client to reconnect to the origin
@@ -465,9 +484,13 @@ class AsyncAuth(object):
         while True:
             try:
                 creds = yield self.sign_in(channel=channel)
-            except SaltClientError as error:
+            except SaltClientError as exc:
+                error = exc
                 break
             if creds == 'retry':
+                if self.opts.get('detect_mode') is True:
+                    error = SaltClientError('Detect mode is on')
+                    break
                 if self.opts.get('caller'):
                     print('Minion failed to authenticate with the master, '
                           'has the minion key been accepted?')
@@ -481,6 +504,8 @@ class AsyncAuth(object):
                 continue
             break
         if not isinstance(creds, dict) or 'aes' not in creds:
+            if self.opts.get('detect_mode') is True:
+                error = SaltClientError('-|RETRY|-')
             try:
                 del AsyncAuth.creds_map[self.__key(self.opts)]
             except KeyError:
@@ -547,7 +572,10 @@ class AsyncAuth(object):
             if safe:
                 log.warning('SaltReqTimeoutError: {0}'.format(e))
                 raise tornado.gen.Return('retry')
-            raise SaltClientError('Attempt to authenticate with the salt master failed with timeout error')
+            if self.opts.get('detect_mode') is True:
+                raise tornado.gen.Return('retry')
+            else:
+                raise SaltClientError('Attempt to authenticate with the salt master failed with timeout error')
         if 'load' in payload:
             if 'ret' in payload['load']:
                 if not payload['load']['ret']:
@@ -591,7 +619,7 @@ class AsyncAuth(object):
                 'Salt Minion.\nThe master public key can be found '
                 'at:\n{1}'.format(salt.version.__version__, m_pub_fn)
             )
-            raise SaltSystemExit('Invalid master key')
+            raise SaltClientError('Invalid master key')
         if self.opts.get('syndic_master', False):  # Is syndic
             syndic_finger = self.opts.get('syndic_finger', self.opts.get('master_finger', False))
             if syndic_finger:
@@ -707,11 +735,17 @@ class AsyncAuth(object):
                 except Exception:
                     return '', ''
                 digest = hashlib.sha256(key_str).hexdigest()
+                if six.PY3:
+                    digest = salt.utils.to_bytes(digest)
                 m_digest = public_decrypt(mkey.publickey(), payload['sig'])
                 if m_digest != digest:
                     return '', ''
         else:
             return '', ''
+
+        if six.PY3:
+            key_str = salt.utils.to_str(key_str)
+
         if '_|-' in key_str:
             return key_str.split('_|-')
         else:
@@ -766,7 +800,7 @@ class AsyncAuth(object):
                 m_pub_fn = os.path.join(self.opts['pki_dir'], self.mpub)
                 uid = salt.utils.get_uid(self.opts.get('user', None))
                 with salt.utils.fpopen(m_pub_fn, 'wb+', uid=uid) as wfh:
-                    wfh.write(payload['pub_key'])
+                    wfh.write(salt.utils.to_bytes(payload['pub_key']))
                 return True
             else:
                 log.error('Received signed public-key from master {0} '
@@ -862,7 +896,8 @@ class AsyncAuth(object):
         m_pub_fn = os.path.join(self.opts['pki_dir'], self.mpub)
         m_pub_exists = os.path.isfile(m_pub_fn)
         if m_pub_exists and master_pub and not self.opts['open_mode']:
-            local_master_pub = salt.utils.fopen(m_pub_fn).read()
+            with salt.utils.fopen(m_pub_fn) as fp_:
+                local_master_pub = fp_.read()
 
             if payload['pub_key'].replace('\n', '').replace('\r', '') != \
                     local_master_pub.replace('\n', '').replace('\r', ''):
@@ -912,7 +947,8 @@ class AsyncAuth(object):
                 if not m_pub_exists:
                     # the minion has not received any masters pubkey yet, write
                     # the newly received pubkey to minion_master.pub
-                    salt.utils.fopen(m_pub_fn, 'wb+').write(payload['pub_key'])
+                    with salt.utils.fopen(m_pub_fn, 'wb+') as fp_:
+                        fp_.write(salt.utils.to_bytes(payload['pub_key']))
                 return self.extract_aes(payload, master_pub=False)
 
     def _finger_fail(self, finger, master_key):
@@ -973,7 +1009,10 @@ class SAuth(AsyncAuth):
         :rtype: Auth
         '''
         self.opts = opts
-        self.token = Crypticle.generate_key_string()
+        if six.PY2:
+            self.token = Crypticle.generate_key_string()
+        else:
+            self.token = salt.utils.to_bytes(Crypticle.generate_key_string())
         self.serial = salt.payload.Serial(self.opts)
         self.pub_path = os.path.join(self.opts['pki_dir'], 'minion.pub')
         self.rsa_path = os.path.join(self.opts['pki_dir'], 'minion.pem')
@@ -1147,7 +1186,7 @@ class Crypticle(object):
     Signing algorithm: HMAC-SHA256
     '''
 
-    PICKLE_PAD = 'pickle::'
+    PICKLE_PAD = b'pickle::'
     AES_BLOCK_SIZE = 16
     SIG_SIZE = hashlib.sha256().digest_size
 
@@ -1167,7 +1206,10 @@ class Crypticle(object):
 
     @classmethod
     def extract_keys(cls, key_string, key_size):
-        key = key_string.decode('base64')
+        if six.PY2:
+            key = key_string.decode('base64')
+        else:
+            key = salt.utils.to_bytes(base64.b64decode(key_string))
         assert len(key) == key_size / 8 + cls.SIG_SIZE, 'invalid key'
         return key[:-cls.SIG_SIZE], key[-cls.SIG_SIZE:]
 
@@ -1177,7 +1219,10 @@ class Crypticle(object):
         '''
         aes_key, hmac_key = self.keys
         pad = self.AES_BLOCK_SIZE - len(data) % self.AES_BLOCK_SIZE
-        data = data + pad * chr(pad)
+        if six.PY2:
+            data = data + pad * chr(pad)
+        else:
+            data = data + salt.utils.to_bytes(pad * chr(pad))
         iv_bytes = os.urandom(self.AES_BLOCK_SIZE)
         cypher = AES.new(aes_key, AES.MODE_CBC, iv_bytes)
         data = iv_bytes + cypher.encrypt(data)
@@ -1196,8 +1241,13 @@ class Crypticle(object):
             log.debug('Failed to authenticate message')
             raise AuthenticationError('message authentication failed')
         result = 0
-        for zipped_x, zipped_y in zip(mac_bytes, sig):
-            result |= ord(zipped_x) ^ ord(zipped_y)
+
+        if six.PY2:
+            for zipped_x, zipped_y in zip(mac_bytes, sig):
+                result |= ord(zipped_x) ^ ord(zipped_y)
+        else:
+            for zipped_x, zipped_y in zip(mac_bytes, sig):
+                result |= zipped_x ^ zipped_y
         if result != 0:
             log.debug('Failed to authenticate message')
             raise AuthenticationError('message authentication failed')
@@ -1205,7 +1255,10 @@ class Crypticle(object):
         data = data[self.AES_BLOCK_SIZE:]
         cypher = AES.new(aes_key, AES.MODE_CBC, iv_bytes)
         data = cypher.decrypt(data)
-        return data[:-ord(data[-1])]
+        if six.PY2:
+            return data[:-ord(data[-1])]
+        else:
+            return data[:-data[-1]]
 
     def dumps(self, obj):
         '''
@@ -1213,7 +1266,7 @@ class Crypticle(object):
         '''
         return self.encrypt(self.PICKLE_PAD + self.serial.dumps(obj))
 
-    def loads(self, data):
+    def loads(self, data, raw=False):
         '''
         Decrypt and un-serialize a python object
         '''
@@ -1221,4 +1274,5 @@ class Crypticle(object):
         # simple integrity check to verify that we got meaningful data
         if not data.startswith(self.PICKLE_PAD):
             return {}
-        return self.serial.loads(data[len(self.PICKLE_PAD):])
+        load = self.serial.loads(data[len(self.PICKLE_PAD):], raw=raw)
+        return load
