@@ -310,6 +310,39 @@ def sync_states(saltenv=None, refresh=True):
     return ret
 
 
+def refresh_grains(**kwargs):
+    '''
+    .. versionadded:: 2016.3.6,2016.11.4,Nitrogen
+
+    Refresh the minion's grains without syncing custom grains modules from
+    ``salt://_grains``.
+
+    .. note::
+        The available execution modules will be reloaded as part of this
+        proceess, as grains can affect which modules are available.
+
+    refresh_pillar : True
+        Set to ``False`` to keep pillar data from being refreshed.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.refresh_grains
+    '''
+    kwargs = salt.utils.clean_kwargs(**kwargs)
+    _refresh_pillar = kwargs.pop('refresh_pillar', True)
+    if kwargs:
+        salt.utils.invalid_kwargs(kwargs)
+    # Modules and pillar need to be refreshed in case grains changes affected
+    # them, and the module refresh process reloads the grains and assigns the
+    # newly-reloaded grains to each execution module's __grains__ dunder.
+    refresh_modules()
+    if _refresh_pillar:
+        refresh_pillar()
+    return True
+
+
 def sync_grains(saltenv=None, refresh=True):
     '''
     .. versionadded:: 0.10.0
@@ -1155,6 +1188,7 @@ def runner(name, **kwargs):
     '''
     jid = kwargs.pop('__orchestration_jid__', None)
     saltenv = kwargs.pop('__env__', 'base')
+    full_return = kwargs.pop('full_return', False)
     kwargs = salt.utils.clean_kwargs(**kwargs)
 
     if 'master_job_cache' not in __opts__:
@@ -1178,7 +1212,10 @@ def runner(name, **kwargs):
             prefix='run'
         )
 
-    return rclient.cmd(name, kwarg=kwargs, print_event=False, full_return=True)
+    return rclient.cmd(name,
+                       kwarg=kwargs,
+                       print_event=False,
+                       full_return=full_return)
 
 
 def wheel(name, *args, **kwargs):
